<template>
    ${P.getPartCode(item,'PANEL').code}
</template>
<#assign import_block>
import { PanelDetailModel,PanelRawitemModel,PanelTabPanelModel,PanelTabPageModel,PanelFieldModel,PanelContainerModel,PanelControlModel,PanelUserControlModel,PanelButtonModel } from '@/model/panel-detail';
import ${srfclassname('${ctrl.codeName}')}Model from './${srffilepath2(ctrl.codeName)}-${ctrl.getControlType()?lower_case}-model';
import CodeListService from "@/codelist/codelist-service";
import UIService from '@/uiservice/ui-service';
</#assign>
<#ibizinclude>
../@MACRO/CONTROL/CONTROL_HEADER-BASE.vue.ftl
</#ibizinclude>
<#ibizinclude>
../../@MACRO/PANEL.ftl
</#ibizinclude>
    /**
     * 接口实现
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    getDatas(): any[] {
        if (!this.layoutData) {
            return [];
        }
        return [this.layoutData];
    }

    /**
     * 接口实现
     *
     * @returns {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    getData() {
        return this.layoutData;
    }

    /**
     * 父级部件引用
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public parentRef?: any;

    /**
     * 面板数据对象
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public inputData?: any;

    /**
     * 操作栏模型数据
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Prop() public actionModel?: any;

    /**
     * UI数据对象
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public data:any = {};

    /**
     * 数据模型对象
     *
     * @type {${srfclassname('${ctrl.codeName}')}Model}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public dataModel:${srfclassname('${ctrl.codeName}')}Model = new ${srfclassname('${ctrl.codeName}')}Model();

    /**
     * 代码表服务对象
     *
     * @type {CodeListService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public codeListService:CodeListService = new CodeListService();

    <#if appde??>
    /**
     * 界面UI服务对象
     *
     * @type {${srfclassname('${appde.getCodeName()}')}UIService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */  
    public appUIService:${srfclassname('${appde.getCodeName()}')}UIService = new ${srfclassname('${appde.getCodeName()}')}UIService();
    <#else>
    /**
     * 界面UI服务对象
     *
     * @type {UIService}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */  
    public appUIService:UIService = new UIService();
    </#if>

    /**
     * 视图布局顶级成员名称
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public rootLayoutDetailNames: string[] = <@compress  single_line=true>[
        <#if ctrl.getRootPSPanelItems?? && ctrl.getRootPSPanelItems()??>
        <#list ctrl.getRootPSPanelItems() as panelItem>
        '${panelItem.name}'<#if panelItem_has_next>,</#if>
        </#list>
        </#if>
        ];</@compress>

<#--  <@panelBase ctrl>  -->

    /**
     * 视图布局面板项模型对象
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public layoutItems:any = {
    <#list ctrl.getAllPSPanelItems() as panelItem>
        <#--  <@getPanelItem panelItem 'ITEMLAYOUT' />  -->
    </#list>
    };

    /**
     * 值规则对象
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public rules:any={
    <#if ctrl.getAllPSPanelItems()??>
    <#list ctrl.getAllPSPanelItems() as panelItem>
    <#if panelItem.getItemType() == "FIELD">
        ${panelItem.getName()}:[
            { required: <#if panelItem.isAllowEmpty()>false<#else>true</#if>, type: 'string', message: '${panelItem.getCaption()} 值不能为空'},
            { required: <#if panelItem.isAllowEmpty()>false<#else>true</#if>, type: 'string', message: '${panelItem.getCaption()} 值不能为空'}
        ],
    </#if>
    </#list>
    </#if>
    };

    /**
     * 监听数据对象
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    @Watch('inputData',{immediate:true,deep: true})
    public onInputDataChange(newVal: any, oldVal: any){
        if(newVal){
            this.computedUIData(newVal);
            this.layoutData = Util.deepCopy(newVal);
            this.computeButtonState(newVal);
            this.panelLogic('');
            this.$forceUpdate();
        }
    }

    /**
     * 生命周期
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public mounted () {
        this.afterMounted();
    }

    /**
     * 执行mounted后的逻辑
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public afterMounted () {
        const _this: any = this;
        _this.initLayout().then((result: any) => {
            _this.isLayoutLoadding = false;

        });
    }

    /**
     * 计算UI展示数据
     * 
     * @param codelistArray 代码表模型数组
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public computedUIData(newVal:any){
        if((this.dataModel.getDataItems instanceof Function) && this.dataModel.getDataItems().length >0){
            this.dataModel.getDataItems().forEach((item:any) =>{
                this.data[item.name] = newVal[item.prop];
            })
        }
    }

    /**
     * 计算面板按钮权限状态
     *
     * @param {*} [data] 传入数据
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public computeButtonState(data:any){
        // 若为项布局面板，存在parentRef
        if(this.parentRef){
            let targetData:any = this.parentRef.transformData(data);
            if(this.layoutModelDetails && Object.keys(this.layoutModelDetails).length >0){
                Object.keys(this.layoutModelDetails).forEach((name:any) =>{
                    if(this.layoutModelDetails[name] && this.layoutModelDetails[name].uiAction && this.layoutModelDetails[name].uiAction.dataAccaction && Object.is(this.layoutModelDetails[name].itemType,"BUTTON")){
                        let tempUIAction:any = JSON.parse(JSON.stringify(this.layoutModelDetails[name].uiAction));
                        let result: any[] = ViewTool.calcActionItemAuthState(targetData, [tempUIAction], this.appUIService ? this.appUIService : null);
                        this.layoutModelDetails[name].visible = tempUIAction.visabled;
                        this.layoutModelDetails[name].disabled = tempUIAction.disabled;
                    }
                })
            }
        }
    }

    /**
     * 打开编辑数据视图
     *
     * @type {any}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public opendata(args: any[],fullargs?:any[],params?: any, $event?: any, xData?: any){
        if (this.parentRef.opendata && this.parentRef.opendata instanceof Function) {
            this.parentRef.opendata(args,fullargs,params, $event, xData);
        }
    }

    /**
     * 打开新建数据视图
     *
     * @type {any}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public newdata(args: any[],fullargs?:any[], params?: any, $event?: any, xData?: any) {
        if (this.parentRef.newdata && this.parentRef.newdata instanceof Function) {
            this.parentRef.newdata(args,fullargs,params, $event, xData);
        }
    }
    
    /**
     * 删除
     *
     * @param {any[]} datas
     * @returns {Promise<any>}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public async remove(datas: any[]): Promise<any> {
        if (this.parentRef.remove && this.parentRef.remove instanceof Function) {
            return this.parentRef.remove(datas);
        }
    }

    /**
     * 刷新
     *
     * @param {*} [args={}]
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public refresh(args: any = {}) {
        if (this.parentRef.refresh && this.parentRef.refresh instanceof Function) {
            this.parentRef.refresh(args);
        }
    }

    /**
     * 处理值改变
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public handleValueChange(args: { name: string, value: any }) {
        if (!args || !args.name || Object.is(args.name, '') || !this.layoutData.hasOwnProperty(args.name)) {
            return;
        }
        const { name, value } = args;
        this.data[name] = value;
        this.layoutData[name] = value;
        this.layoutModelDetails[name].setData(value);
        this.panelLogic(name);
        this.panelEditItemChange(this.data, name, value);
    }

    /**
     * 面板编辑项值变化后续操作
     *
     * @public
     * @param data 面板数据
     * @param property 编辑项名
     * @param value 编辑项值
     * @returns {void}
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public panelEditItemChange(data: any, property: string, value: any){
        // 面板数据变化事件
        if((this.dataModel.getDataItems instanceof Function) && this.dataModel.getDataItems().length >0){
            let modelitem =this.dataModel.getDataItems().find((item:any) =>{
                return item.name === property;
            }) 
            if(modelitem){
                this.$emit('panelDataChange',{[modelitem.prop]: value});
            }
        }
    }

    /**
     * 面板逻辑
     *
     * @public
     * @param {{ name: string, newVal: any, oldVal: any }} { name, newVal, oldVal }
     * @memberof ${srfclassname('${ctrl.codeName}')}Base
     */
    public panelLogic(name: string): void {
        <#list ctrl.getAllPSPanelItems() as panelItem>
            <@panelItemLogic panelItem>
        </#list>
    }

    /**
     * 获取按钮行为xData
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public getButtonXData(name: string): any {
        let xData = null;
        <@xData />
        return xData;
    }

    /**
     * 处理按钮点击
     *
     * @public
     * @memberof ${srfclassname('${ctrl.name}')}Base
     */
    public handleButtonClick(name: string, $event?: any) {
        const datas: any[] = [this.layoutData];
        const xData: any = this.getButtonXData(name);
        const paramJO: any = {};
        const contextJO: any = {};
        const _this: any = this;
        <#list ctrl.getAllPSPanelItems() as panelItem>
        <#if panelItem.getItemType()  == 'BUTTON' && panelItem.getPSUIAction?? && panelItem.getPSUIAction()??>
        if (Object.is(name, '${panelItem.name}')) {
            <#if panelItem.getPSUIAction().getPSAppDataEntity?? && panelItem.getPSUIAction().getPSAppDataEntity()?? && panelItem.getPSUIAction().getUIActionMode()?? && (panelItem.getPSUIAction().getUIActionMode() == "FRONT" || panelItem.getPSUIAction().getUIActionMode() == "BACKEND" || panelItem.getPSUIAction().getUIActionMode() == "WFFRONT" || panelItem.getPSUIAction().getUIActionMode() == "WFBACKEND")>
            <#assign curAppEntity = panelItem.getPSUIAction().getPSAppDataEntity() />
            const curUIService: ${srfclassname('${curAppEntity.codeName}')}UIService  = new ${srfclassname('${curAppEntity.codeName}')}UIService();
            curUIService.${panelItem.getPSUIAction().getFullCodeName()}(datas, contextJO, paramJO,  $event, xData, this, undefined);
            <#else>
            _this.${panelItem.getPSUIAction().getFullCodeName()}(datas, contextJO, paramJO,  $event, xData, this, undefined);
            </#if>
        }
        </#if>
        </#list>
    }

    <#if ctrl.getPSUIActions?? && ctrl.getPSUIActions()??>
    <#list ctrl.getPSUIActions() as uiAction>
    <#if !P.exists("importService", uiAction.getFullCodeName(), "")>
    <#if !(uiAction.getPSAppDataEntity?? && uiAction.getPSAppDataEntity()??) || (uiAction.getPSAppDataEntity?? && uiAction.getPSAppDataEntity()?? && uiAction.getUIActionMode?? && (uiAction.getUIActionMode() == 'SYS' || uiAction.getUIActionMode() == 'CUSTOM'))>
    <@viewLayoutPanelUIAction item=uiAction />
    </#if>
    </#if>
    </#list>
    </#if>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>

